@@ -56,26 +56,23 @@ to be an installation dependency for our packages yet--it is still too unstable
 # irritating Python bug that can crop up when using ./setup.py test.
 # See: http://www.eby-sarna.com/pipermail/peak/2010-May/003355.html
 try:
-    import multiprocessing  # flake8: noqa
+    import multiprocessing  # noqa
 except ImportError:
     pass
-import logging  # flake8: noqa
+import logging  # noqa
 
+from collections import defaultdict
 import os
 import re
 import sys
 import traceback
 
-from collections import defaultdict
-
 import distutils.ccompiler
-
+from distutils import errors
 from distutils import log
-from distutils.errors import (DistutilsOptionError, DistutilsModuleError,
-                              DistutilsFileError)
-from setuptools.command.egg_info import manifest_maker
-from setuptools.dist import Distribution
-from setuptools.extension import Extension
+import pkg_resources
+from setuptools import dist as st_dist
+from setuptools import extension
 
 try:
     import ConfigParser as configparser
@@ -101,10 +98,12 @@ D1_D2_SETUP_ARGS = {
     "maintainer": ("metadata",),
     "maintainer_email": ("metadata",),
     "url": ("metadata", "home_page"),
+    "project_urls": ("metadata",),
     "description": ("metadata", "summary"),
     "keywords": ("metadata",),
     "long_description": ("metadata", "description"),
-    "download-url": ("metadata",),
+    "long_description_content_type": ("metadata", "description_content_type"),
+    "download_url": ("metadata",),
     "classifiers": ("metadata", "classifier"),
     "platforms": ("metadata", "platform"),  # **
     "license": ("metadata",),
@@ -112,6 +111,7 @@ D1_D2_SETUP_ARGS = {
     # broken distutils requires
     "install_requires": ("metadata", "requires_dist"),
     "setup_requires": ("metadata", "setup_requires_dist"),
+    "python_requires": ("metadata",),
     "provides": ("metadata", "provides_dist"),  # **
     "obsoletes": ("metadata", "obsoletes_dist"),  # **
     "package_dir": ("files", 'packages_root'),
@@ -148,6 +148,9 @@ MULTI_FIELDS = ("classifiers",
                 "tests_require",
                 "cmdclass")
 
+# setup() arguments that can have mapping values in setup.cfg
+MAP_FIELDS = ("project_urls",)
+
 # setup() arguments that contain boolean values
 BOOL_FIELDS = ("use_2to3", "zip_safe", "include_package_data")
 
@@ -187,28 +190,38 @@ def resolve_name(name):
     return ret
 
 
-def cfg_to_args(path='setup.cfg'):
-    """ Distutils2 to distutils1 compatibility util.
+def cfg_to_args(path='setup.cfg', script_args=()):
+    """Distutils2 to distutils1 compatibility util.
 
-        This method uses an existing setup.cfg to generate a dictionary of
-        keywords that can be used by distutils.core.setup(kwargs**).
-
-        :param file:
-            The setup.cfg path.
-        :raises DistutilsFileError:
-            When the setup.cfg file is not found.
+    This method uses an existing setup.cfg to generate a dictionary of
+    keywords that can be used by distutils.core.setup(kwargs**).
 
+    :param path:
+        The setup.cfg path.
+    :param script_args:
+        List of commands setup.py was called with.
+    :raises DistutilsFileError:
+        When the setup.cfg file is not found.
     """
 
     # The method source code really starts here.
-    parser = configparser.SafeConfigParser()
+    if sys.version_info >= (3, 2):
+            parser = configparser.ConfigParser()
+    else:
+            parser = configparser.SafeConfigParser()
     if not os.path.exists(path):
-        raise DistutilsFileError("file '%s' does not exist" %
-                                 os.path.abspath(path))
-    parser.read(path)
+        raise errors.DistutilsFileError("file '%s' does not exist" %
+                                        os.path.abspath(path))
+    try:
+        parser.read(path, encoding='utf-8')
+    except TypeError:
+        # Python 2 doesn't accept the encoding kwarg
+        parser.read(path)
     config = {}
     for section in parser.sections():
-        config[section] = dict(parser.items(section))
+        config[section] = dict()
+        for k, value in parser.items(section):
+            config[section][k.replace('-', '_')] = value
 
     # Run setup_hooks, if configured
     setup_hooks = has_get_option(config, 'global', 'setup_hooks')
@@ -228,11 +241,11 @@ def cfg_to_args(path='setup.cfg'):
                 if hook != 'pbr.hooks.setup_hook']
             for hook in setup_hooks:
                 hook_fn = resolve_name(hook)
-                try :
+                try:
                     hook_fn(config)
                 except SystemExit:
                     log.error('setup hook %s terminated the installation')
-                except:
+                except Exception:
                     e = sys.exc_info()[1]
                     log.error('setup hook %s raised exception: %s\n' %
                               (hook, e))
@@ -242,7 +255,7 @@ def cfg_to_args(path='setup.cfg'):
         # Run the pbr hook
         pbr.hooks.setup_hook(config)
 
-        kwargs = setup_cfg_to_setup_kwargs(config)
+        kwargs = setup_cfg_to_setup_kwargs(config, script_args)
 
         # Set default config overrides
         kwargs['include_package_data'] = True
@@ -258,8 +271,6 @@ def cfg_to_args(path='setup.cfg'):
         if entry_points:
             kwargs['entry_points'] = entry_points
 
-        wrap_commands(kwargs)
-
         # Handle the [files]/extra_files option
         files_extra_files = has_get_option(config, 'files', 'extra_files')
         if files_extra_files:
@@ -273,14 +284,16 @@ def cfg_to_args(path='setup.cfg'):
     return kwargs
 
 
-def setup_cfg_to_setup_kwargs(config):
-    """Processes the setup.cfg options and converts them to arguments accepted
+def setup_cfg_to_setup_kwargs(config, script_args=()):
+    """Convert config options to kwargs.
+
+    Processes the setup.cfg options and converts them to arguments accepted
     by setuptools' setup() function.
     """
 
     kwargs = {}
 
-    # Temporarily holds install_reqires and extra_requires while we
+    # Temporarily holds install_requires and extra_requires while we
     # parse env_markers.
     all_requirements = {}
 
@@ -317,6 +330,12 @@ def setup_cfg_to_setup_kwargs(config):
             in_cfg_value = split_csv(in_cfg_value)
         if arg in MULTI_FIELDS:
             in_cfg_value = split_multiline(in_cfg_value)
+        elif arg in MAP_FIELDS:
+            in_cfg_map = {}
+            for i in split_multiline(in_cfg_value):
+                k, v = i.split('=')
+                in_cfg_map[k.strip()] = v.strip()
+            in_cfg_value = in_cfg_map
         elif arg in BOOL_FIELDS:
             # Provide some flexibility here...
             if in_cfg_value.lower() in ('true', 't', '1', 'yes', 'y'):
@@ -334,12 +353,13 @@ def setup_cfg_to_setup_kwargs(config):
                 # Split install_requires into package,env_marker tuples
                 # These will be re-assembled later
                 install_requires = []
-                requirement_pattern = '(?P<package>[^;]*);?(?P<env_marker>[^#]*?)(?:\s*#.*)?$'
+                requirement_pattern = (
+                    r'(?P<package>[^;]*);?(?P<env_marker>[^#]*?)(?:\s*#.*)?$')
                 for requirement in in_cfg_value:
                     m = re.match(requirement_pattern, requirement)
                     requirement_package = m.group('package').strip()
                     env_marker = m.group('env_marker').strip()
-                    install_requires.append((requirement_package,env_marker))
+                    install_requires.append((requirement_package, env_marker))
                 all_requirements[''] = install_requires
             elif arg == 'package_dir':
                 in_cfg_value = {'': in_cfg_value}
@@ -360,7 +380,7 @@ def setup_cfg_to_setup_kwargs(config):
                         else:
                             prev = data_files[key.strip()] = value.split()
                     elif firstline:
-                        raise DistutilsOptionError(
+                        raise errors.DistutilsOptionError(
                             'malformed package_data first line %r (misses '
                             '"=")' % line)
                     else:
@@ -373,7 +393,7 @@ def setup_cfg_to_setup_kwargs(config):
                 in_cfg_value = data_files
             elif arg == 'cmdclass':
                 cmdclass = {}
-                dist = Distribution()
+                dist = st_dist.Distribution()
                 for cls_name in in_cfg_value:
                     cls = resolve_name(cls_name)
                     cmd = cls(dist)
@@ -394,8 +414,16 @@ def setup_cfg_to_setup_kwargs(config):
     # -> {'fred': ['bar'], 'fred:marker':['foo']}
 
     if 'extras' in config:
-        requirement_pattern = '(?P<package>[^:]*):?(?P<env_marker>[^#]*?)(?:\s*#.*)?$'
+        requirement_pattern = (
+            r'(?P<package>[^:]*):?(?P<env_marker>[^#]*?)(?:\s*#.*)?$')
         extras = config['extras']
+        # Add contents of test-requirements, if any, into an extra named
+        # 'test' if one does not already exist.
+        if 'test' not in extras:
+            from pbr import packaging
+            extras['test'] = "\n".join(packaging.parse_requirements(
+                packaging.TEST_REQUIREMENTS_FILES)).replace(';', ':')
+
         for extra in extras:
             extra_requirements = []
             requirements = split_multiline(extras[extra])
@@ -403,7 +431,7 @@ def setup_cfg_to_setup_kwargs(config):
                 m = re.match(requirement_pattern, requirement)
                 extras_value = m.group('package').strip()
                 env_marker = m.group('env_marker')
-                extra_requirements.append((extras_value,env_marker))
+                extra_requirements.append((extras_value, env_marker))
             all_requirements[extra] = extra_requirements
 
     # Transform the full list of requirements into:
@@ -419,6 +447,22 @@ def setup_cfg_to_setup_kwargs(config):
         for requirement, env_marker in all_requirements[req_group]:
             if env_marker:
                 extras_key = '%s:(%s)' % (req_group, env_marker)
+                # We do not want to poison wheel creation with locally
+                # evaluated markers.  sdists always re-create the egg_info
+                # and as such do not need guarded, and pip will never call
+                # multiple setup.py commands at once.
+                if 'bdist_wheel' not in script_args:
+                    try:
+                        if pkg_resources.evaluate_marker('(%s)' % env_marker):
+                            extras_key = req_group
+                    except SyntaxError:
+                        log.error(
+                            "Marker evaluation failed, see the following "
+                            "error.  For more information see: "
+                            "http://docs.openstack.org/"
+                            "pbr/latest/user/using.html#environment-markers"
+                        )
+                        raise
             else:
                 extras_key = req_group
             extras_require.setdefault(extras_key, []).append(requirement)
@@ -430,9 +474,10 @@ def setup_cfg_to_setup_kwargs(config):
 
 
 def register_custom_compilers(config):
-    """Handle custom compilers; this has no real equivalent in distutils, where
-    additional compilers could only be added programmatically, so we have to
-    hack it in somehow.
+    """Handle custom compilers.
+
+    This has no real equivalent in distutils, where additional compilers could
+    only be added programmatically, so we have to hack it in somehow.
     """
 
     compilers = has_get_option(config, 'global', 'compilers')
@@ -454,7 +499,7 @@ def register_custom_compilers(config):
 
             module_name = compiler.__module__
             # Note; this *will* override built in compilers with the same name
-            # TODO: Maybe display a warning about this?
+            # TODO(embray): Maybe display a warning about this?
             cc = distutils.ccompiler.compiler_class
             cc[name] = (module_name, compiler.__name__, desc)
 
@@ -511,111 +556,29 @@ def get_extension_modules(config):
             if ext_args:
                 if 'name' not in ext_args:
                     ext_args['name'] = labels[1]
-                ext_modules.append(Extension(ext_args.pop('name'),
-                                             **ext_args))
+                ext_modules.append(extension.Extension(ext_args.pop('name'),
+                                                       **ext_args))
     return ext_modules
 
 
 def get_entry_points(config):
-    """Process the [entry_points] section of setup.cfg to handle setuptools
-    entry points.  This is, of course, not a standard feature of
-    distutils2/packaging, but as there is not currently a standard alternative
-    in packaging, we provide support for them.
+    """Process the [entry_points] section of setup.cfg.
+
+    Processes setup.cfg to handle setuptools entry points. This is, of course,
+    not a standard feature of distutils2/packaging, but as there is not
+    currently a standard alternative in packaging, we provide support for them.
     """
 
-    if not 'entry_points' in config:
+    if 'entry_points' not in config:
         return {}
 
     return dict((option, split_multiline(value))
                 for option, value in config['entry_points'].items())
 
 
-def wrap_commands(kwargs):
-    dist = Distribution()
-
-    # This should suffice to get the same config values and command classes
-    # that the actual Distribution will see (not counting cmdclass, which is
-    # handled below)
-    dist.parse_config_files()
-
-    for cmd, _ in dist.get_command_list():
-        hooks = {}
-        for opt, val in dist.get_option_dict(cmd).items():
-            val = val[1]
-            if opt.startswith('pre_hook.') or opt.startswith('post_hook.'):
-                hook_type, alias = opt.split('.', 1)
-                hook_dict = hooks.setdefault(hook_type, {})
-                hook_dict[alias] = val
-        if not hooks:
-            continue
-
-        if 'cmdclass' in kwargs and cmd in kwargs['cmdclass']:
-            cmdclass = kwargs['cmdclass'][cmd]
-        else:
-            cmdclass = dist.get_command_class(cmd)
-
-        new_cmdclass = wrap_command(cmd, cmdclass, hooks)
-        kwargs.setdefault('cmdclass', {})[cmd] = new_cmdclass
-
-
-def wrap_command(cmd, cmdclass, hooks):
-    def run(self, cmdclass=cmdclass):
-        self.run_command_hooks('pre_hook')
-        cmdclass.run(self)
-        self.run_command_hooks('post_hook')
-
-    return type(cmd, (cmdclass, object),
-                {'run': run, 'run_command_hooks': run_command_hooks,
-                 'pre_hook': hooks.get('pre_hook'),
-                 'post_hook': hooks.get('post_hook')})
-
-
-def run_command_hooks(cmd_obj, hook_kind):
-    """Run hooks registered for that command and phase.
-
-    *cmd_obj* is a finalized command object; *hook_kind* is either
-    'pre_hook' or 'post_hook'.
-    """
-
-    if hook_kind not in ('pre_hook', 'post_hook'):
-        raise ValueError('invalid hook kind: %r' % hook_kind)
-
-    hooks = getattr(cmd_obj, hook_kind, None)
-
-    if hooks is None:
-        return
-
-    for hook in hooks.values():
-        if isinstance(hook, str):
-            try:
-                hook_obj = resolve_name(hook)
-            except ImportError:
-                err = sys.exc_info()[1] # For py3k
-                raise DistutilsModuleError('cannot find hook %s: %s' %
-                                           (hook,err))
-        else:
-            hook_obj = hook
-
-        if not hasattr(hook_obj, '__call__'):
-            raise DistutilsOptionError('hook %r is not callable' % hook)
-
-        log.info('running %s %s for command %s',
-                 hook_kind, hook, cmd_obj.get_command_name())
-
-        try :
-            hook_obj(cmd_obj)
-        except:
-            e = sys.exc_info()[1]
-            log.error('hook %s raised exception: %s\n' % (hook, e))
-            log.error(traceback.format_exc())
-            sys.exit(1)
-
-
 def has_get_option(config, section, option):
     if section in config and option in config[section]:
         return config[section][option]
-    elif section in config and option.replace('_', '-') in config[section]:
-        return config[section][option.replace('_', '-')]
     else:
         return False
 
@@ -625,7 +588,7 @@ def split_multiline(value):
 
     value = [element for element in
              (line.strip() for line in value.split('\n'))
-             if element]
+             if element and not element.startswith('#')]
     return value
 
 
@@ -638,45 +601,11 @@ def split_csv(value):
     return value
 
 
-def monkeypatch_method(cls):
-    """A function decorator to monkey-patch a method of the same name on the
-    given class.
-    """
-
-    def wrapper(func):
-        orig = getattr(cls, func.__name__, None)
-        if orig and not hasattr(orig, '_orig'):  # Already patched
-            setattr(func, '_orig', orig)
-            setattr(cls, func.__name__, func)
-        return func
-
-    return wrapper
-
-
 # The following classes are used to hack Distribution.command_options a bit
 class DefaultGetDict(defaultdict):
-    """Like defaultdict, but the get() method also sets and returns the default
-    value.
-    """
+    """Like defaultdict, but get() also sets and returns the default value."""
 
     def get(self, key, default=None):
         if default is None:
             default = self.default_factory()
         return super(DefaultGetDict, self).setdefault(key, default)
-
-
-class IgnoreDict(dict):
-    """A dictionary that ignores any insertions in which the key is a string
-    matching any string in `ignore`.  The ignore list can also contain wildcard
-    patterns using '*'.
-    """
-
-    def __init__(self, ignore):
-        self.__ignore = re.compile(r'(%s)' % ('|'.join(
-                                   [pat.replace('*', '.*')
-                                    for pat in ignore])))
-
-    def __setitem__(self, key, val):
-        if self.__ignore.match(key):
-            return
-        super(IgnoreDict, self).__setitem__(key, val)
